pro shapelets_focus,obj,shapelets_focus,decomp,recomp,beta_guess=beta_guess,nmax_guess=nmax_guess,$
     chi20=chi20,tol_chi2=tol_chi2,tol_beta=tol_beta,th_min_geom=th_min_geom,$
     nmax_ran=nmax_ran,psf=psf,printit=printit,silent=silent

;$Id: shapelets_focus.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; TO DO: Should check through history list at end to find chisq closest to one
;        - we have gone past it on occasions and stopped at something else.
;        Will need to store chi-squareds in history structure to implement this!
;
; April 2002 - Minor additions and PSF deconvolution by R. Massey
; April 2002 - Written by A. Refregier
;
; PURPOSE: find the optimal beta, nmax and centroid for the shapelet
; decomposition, in other words 'shapelets_focus' the shapelet on an object. 
;
; INPUT: obj: object structure from shapelets_extract_pstamp.pro
; OPTIONAL INPUT: beta_guess: guess starting point for beta
;                                    (default: finds guess itself) 
;                 nmax_guess: guess starting point for nmax 
;                                    (default; find guess)
;                 tol_chi2: fractional tolerance for Chi^2 minimisation
;                 tol_beta: fractional tolerance for finding beta
;                 chi20: target value of reduced Chi^2 to achieve 
;                        (default=1.)
;                 th_min_geom: minimum geometrical scale theta_min
;                               (default:.5 pixels) 
;                 nmax_ran: range of allowed nmax values (default:[2,12])
; OUTPUT: shapelets_focus: shapelets_focus structure tracing the search on the beta-nmax plane
;         decomp: decomposition structure for the optimal beta-nmax
;                           values
;         recomp: recomposed image for optimal values
;
; flag:
; 0: OK
; 1: has bounced into theta_min or max wall at some point
; 2: nmax_max reached - shapelets incompletely represent object
; 3: chi2>1 but converged (in nmax search) by flatness limit - still incomplete
; 4: did not converge
; 5: centroid off edge
; 6: fatal error
; NB it can only be one of these. If duplicated, highest number has priority.
;-

; declarations
if not keyword_set(chi20) then chi20=1.
if not keyword_set(tol_beta) then tol_beta=1e-2
si=size(obj.image)
n_data=si(1)*si(2)
chi2_sigma=sqrt(2./n_data)      ; rms for reduced Chi^2
if not keyword_set(tol_chi2) then tol_chi2=chi2_sigma
tol_chi2w=1e-3
;tol_chi2w=tol_chi2
if not keyword_set(nmax_ran) then nmax_ran=[2,12]

; guess startting points if required
if not keyword_set(th_min_geom) then th_min_geom=.5
shapelets_guess_nmaxbeta,obj,guess,th_min_geom=th_min_geom*1.05
       ; 1.05 factor present to ensure that guess is not in forbidden area
if not keyword_set(beta_guess) then beta_guess=guess.beta
if not keyword_set(nmax_guess) then nmax_guess=(guess.nmax>nmax_ran(0))<nmax_ran(1)
x0_guess=guess.x0
if keyword_set(printit) then $
print,'guess: nmax, beta, x0:',nmax_guess,beta_guess,x0_guess(0),x0_guess(1)

; find beta which minimises Chi^2 for the guess nmax
nmax=nmax_guess
beta=beta_guess
x0=x0_guess
shapelets_focus_betax0,obj,nmax,shapelets_focus,decomp,recomp,beta_guess=beta,x0_guess=x0,$
  th_min=th_min_geom,tol=1e-2,psf=psf,printit=printit
beta=shapelets_focus.beta
x0=shapelets_focus.x0

; alternate beta-finder and nmax-finder until tolerance is reached
found=0
iloop=0
while not found do begin
  nmax_old=nmax
  beta_old=beta
  
  ; try and find a better nmax
  shapelets_focus_nmax,obj,beta,shapelets_focus,decomp,recomp,x0=x0,chi20=chi20,/append,$
      tol_chi2=tol_chi2,th_min=th_min_geom,nmax_ran=nmax_ran,$
	 psf=psf,printit=printit
  if shapelets_focus.flag eq 6 then break    ; about to crash. quit before it gets there
  nmax=shapelets_focus.nmax
  if nmax_old eq nmax then found=1 ; exit if nmax has not changed  
  if shapelets_focus.flag eq 3 then found=1  ; exit if we`ve found a plateau 
       ; this helps low S/N and I THINK its OK about falling off edge, but...

  ; try and find a better beta
  if not found then begin   
    shapelets_focus_betax0,obj,nmax,shapelets_focus,decomp,recomp,beta_guess=beta,x0_guess=x0,$
       /append,psf=psf,printit=printit;,th_min=th_min_geom,tol=tol_chi2w
    beta=shapelets_focus.beta
    x0=shapelets_focus.x0
    if abs(beta-beta_old) le tol_beta then found=1  ; exit if beta has not changed  
  endif
  if shapelets_focus.flag eq 5 then break    ; if its fallen off edge just quit now

  ; limit number of loops
  iloop=iloop+1
  if iloop gt 20 then begin
    shapelets_focus.flag=(shapelets_focus.flag>4)
    break
  endif
  
endwhile

; adjust shapelets_focus structure
shapelets_focus.beta_guess=beta_guess
shapelets_focus.nmax_guess=nmax_guess
shapelets_focus.x0_guess=x0_guess
shapelets_focus.tol_chi2=tol_chi2
shapelets_focus.chi20=chi20
shapelets_focus.th_min=th_min_geom

; print result 
message,'Flags: '+strtrim(obj.flag,1)+','+strtrim(shapelets_focus.flag,1)+$
       ' n_max='+strtrim(shapelets_focus.nmax,1)+$
        ' beta='+strtrim(shapelets_focus.beta,1)+$
       ' chi^2='+strtrim(shapelets_focus.chi2,1)$
      ;   +' x0=('+strtrim(shapelets_focus.x0(0),1)+','+strtrim(shapelets_focus.x0(1),1)+')' $
                ,/info,noprint=silent


end


