;$Id: shapelets_make_ls_matrix.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
; ***********************************************************************
; ***********************************************************************

pro shapelets_add_zeros,matrix,matrix2

; PURPOSE:
;       Add back in two rows of zeros to the covariance matrix if
;       a_01 and a_10 had been contrained to be zero (probably using
;       shapelets_remove_zeros below)
;
; INPUTS:
;       matrix  - the (as yet incomplete) SQUARE covariance matrix
;                 (MatrixI in shapelets_decomp.pro)
;       matrix2 - the matrix of basis functions and pixel values
;
; OUTPUTS:
;       matrix  - the same matrix, but with two extra rows and two
;                 extra columns of zeros
;
; MODIFICATION HISTORY:
;       Sep 03 - Combined with shapelets_make_ls_matrix.pro
;       Apr 02 - Second matrix added, to allow reconstruction of image
;       Nov 01 - Written by R. Massey
;

COMPILE_OPT idl2, HIDDEN

n=size(matrix)
n=n[1]+2

; add (easy) horizontal row
zero=fltarr(2,n-2)
matrix=[matrix[0,*],zero,matrix[1:*,*]]

;add (difficult) vertical column - must be a better notation
;like zero=fltarr(n,2) etc...? SEE BELOW!

matrix_new=fltarr(n,n)
for i=0,n-1 do begin
 matrix_new[i,0]=matrix[i,0]
 matrix_new[i,1]=0.
 matrix_new[i,2]=0.
 for j=3,n-1 do matrix_new[i,j]=matrix[i,j-2]
endfor
matrix=matrix_new

; add (difficult) vertical column to second matrix
; nice new method is to transpose the matrix, do the same as before,then
;  transpose back!
n=total(size(matrix2)*[0,1])
zero=fltarr(2,n)
matrix2=transpose([transpose(matrix2[*,0]),zero,transpose(matrix2[*,1:*])])

end




; ***********************************************************************
; ***********************************************************************

pro shapelets_extend_zeros,matrix,sky

; PURPOSE:
; Extend the PSF matrix to allow sky fitting in shapelets_decomp.pro
; Sky is a flag (1=flat constant, 2=plane). Result for the latter will be:
;
;     [              |   ]
;     [     P_nm     | 0 ]
;     [              |   ]
;     [--------------|---]
;     [              |100]
;     [       0      |010]
;     [              |001]

COMPILE_OPT idl2, HIDDEN

if sky eq 1 then n=1 else n=3

msize=size(matrix)
h=msize[1]
v=msize[2]

newmatrix=fltarr(h+n,v+n)

for i=0,h-1 do begin
  for j=0,v-1 do begin
    newmatrix[i,j]=matrix[i,j]
  endfor
endfor

for i=0,n-1 do newmatrix[h+i,v+i]=1.

matrix=newmatrix

return

end



; ***********************************************************************
; ***********************************************************************

pro shapelets_remove_zeros,matrix,trans=trans

; PURPOSE:
; Remove two columns from the PSF matrix which correspond to n=1 states.
; These will then not be fit in shapelets_decomp.pro.
; /trans takes transpose first, to remove rows instead of columns.
; Result will look like:
;
;     [                   ]
;     [  P_nm or MatrixT  ]
;     [                   ]
;      | XX        |
;      | XX        |
;      V XX        V
;     [ ]  [              ]
;     [ ]..[     P_nm     ]
;     [ ]  [              ]
;       \        |
;        \ 	     |
;        _\|     V
;     [   		      ]
;     [     P_nm      ]
;     [   		      ]

COMPILE_OPT idl2, HIDDEN

if keyword_set(trans) then matrix=transpose(matrix)

  matrix[2,*]=matrix[0,*]
  matrix=matrix[2:*,*]

if keyword_set(trans) then matrix=transpose(matrix)

return

end








; ***********************************************************************
; ***********************************************************************

pro shapelets_make_ls_matrix, Matrix, n_max, beta, nf1, nf2, x0, non1=non1, $
                 sky=sky, noskygrad=noskygrad, integrate=integrate, psf=psf
;+
; PURPOSE:
;       Create an array with which to compute the least-squares linear algebra
;       fit of shapelets basis functions to data. Can also use this array to
;       perform the overlap integrals calculation.
;
; CATEGORY:
;       Linear algebra.
;
; CALLING PROCEDURE:
;       shapelets_make_ls_matrix, MatrixI, n_max, w, nf1, nf2, x0
;
; INPUTS:
;       n_max - maximum order of the hermite ceofficients to compute
;       beta  - scalar basis function scale size
;       nf1   - integer number of pixels in x direction
;       nf2   - integer number of pixels in y direction
;       x0    - [x,y] vector centre coordinates of the basis functions
;       psf   - Cartesian shapelet decomp structure of the local PSF
;               for deconvolution
;
; KEYWORD PARAMETERS:
;       /INT  - use basis functions integrated within square pixels
;       /SKY  - fit sky background with plane
;       /NOSKY- fit sky with just a constant
;
; OUTPUTS:
;       Matrix- least squares fitting matrix
;
; NOTES:
;       Perhaps it would have been easier to just pass the grid coordinate
;       positions of pixel centres (usually called x1 and x2), rather than
;       the number of pixels and their centre. The grids now have to be
;       reconstructed on the first line. Oh well.
;
; MODIFICATION HISTORY:
;       Sep 03 - Other routines involving the addition or subtraction of
;                rows contaiing zeros (to constrain some fit parameters)
;                combined into one file by RM
;       Nov 01 - Written by R. Massey
;-

; construct x-y position array
shapelets_make_xarr, [nf1, nf2], x1, x2, x0=x0

; work out ns and if necessary add extra fitting parameter for sky background
n_pix=nf1*nf2
shapelets_make_nvec, n_max, n1, n2, n_coeffs
n_coeffs_tot=n_coeffs
if keyword_set(sky) then begin
  n_coeffs_tot=n_coeffs_tot+1
  if ( sky eq 2 ) then n_coeffs_tot=n_coeffs_tot+2
endif

; create LS matrix
Matrix=fltarr(n_coeffs_tot, n_pix, /nozero)

; calculate basis function arrays
Basis=shapelets_phi([n_max,0],x1/beta,x2/beta,integrate=integrate,/array)/beta

; put basis functions into Matrix
j=0L                         ; loop over all pixels in image
for f2i=0,nf2-1 do begin
  for f1i=0,nf1-1 do begin
    for i=0L,n_coeffs-1 do begin   ; loop over all basis functions
      Matrix[i,j]=Basis[f1i,f2i,i]
    endfor
    j=j+1
  endfor
endfor

; add extra basis functions to map sky background
if keyword_set(sky) then begin
  Matrix(n_coeffs,*)=1.                   ; constant
  if ( sky eq 2 ) then begin
  j=0L
    for f2i=0,nf2-1 do begin ; loop over all pixels in image
      for f1i=0,nf1-1 do begin
        Matrix(n_coeffs+1,j)=x1(f1i,f2i)  ; x (for a plane)
        Matrix(n_coeffs+2,j)=x2(f1i,f2i)  ; y
        j=j+1
      endfor
    endfor
  endif
endif

; remove some basis functions if we are requiring a_01 and a_10 to be zero
if keyword_set(non1) and n_max ge 2 then shapelets_remove_zeros,Matrix

end

