pro shapelets_dilate, decomp, scaling, polar=polar, cartesian=cartesian, n_max=n_max, flux=flux

;$Id: shapelets_dilate.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Richard Massey - February 2002
;
; TO DO - check if kappa is calculated correctly (kappa=[scaling-1]/2) ?
;
; Dilates (kappa operator) an object. There are three methods for this:
; (1) To first order, in Cartesian shapelets, using a&a^ ladder operators.
; (2) Convert to polar shapelets and use ladder operators, but again only to
;     first order. Then convert back to Cartesian coefficients.
; (3) An alternative method uses the C_nml formalism for convolution.
;     Convolution with a delta function (ie a PSF with beta -> 0) leaves the
;     object unchanged, but we can now choose a new beta (in shapelets_convolution_matrix.pro,
;     the notation is to change alpha to gamma). This is a general case, no
;     longer limited to first order.
; Option three is the default; the others are selected using switches.
;
; INPUT: decomp:  A shapelet decomposition (unaffected except for coeffs)
;        scaling: kappa factor:
;                 <1 Object shrinks
;                 =1 Object stays the same
;                 >1 Object is enlarged
;        [n_max]: Final result is up to n_max - currently only works with
;                 convolution method only, but could be implemented with others
;        [/flux]: Conserve flux rather than surface brightness as in Grav. lensing
;        [/cart]: Perform first order ladder operations on Cartesian shapelets
;       [/polar]: Perform ladder operations in polar shapelet space
;        DEFAULT: To general order using C_nml convolution formalism
;
; OUTPUT: decomp is returned, still as a Cartesian decomp structure and with
;         the same beta as before, but rescaled
;-

kappa=(scaling-1)/2.

if keyword_set(cartesian) then begin

  ; Perform dilation using Cartesian step operators
  n1=decomp.n1
  n2=decomp.n2
  new_coeffs=decomp.coeffs*(1.+kappa)
  for i=0,decomp.n_coeffs-1 do begin

    ; Expand leftwards in Cartesian shapelet coefficient space
    j=where(n1 eq n1(i)+2 and n2 eq n2(i))
    if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)+kappa/2.*sqrt((n1(i)+1)*(n1(i)+2))*decomp.coeffs(j(0))

    ; Expand downwards in Cartesian shapelet coefficient space
    j=where(n1 eq n1(i) and n2 eq n2(i)+2)
    if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)+kappa/2.*sqrt((n2(i)+1)*(n2(i)+2))*decomp.coeffs(j(0))

    ; Expand rightwards in Cartesian shapelet coefficient space
    j=where(n1 eq n1(i)-2 and n2 eq n2(i))
    if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)+kappa/2.*sqrt(n1(i)*(n1(i)-1))*decomp.coeffs(j(0))

    ; Expand upwards in Cartesian shapelet coefficient space
    j=where(n1 eq n1(i) and n2 eq n2(i)-2)
    if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)+kappa/2.*sqrt(n2(i)*(n2(i)-1))*decomp.coeffs(j(0))

  endfor

  decomp.coeffs=new_coeffs

  ; Conserve flux if asked
  if keyword_set(flux) then decomp.coeffs=decomp.coeffs/scaling


endif else if keyword_set(polar) then begin

  ; Convert to polar shapelet coefficients
  polar_coeffs=shapelets_c2p(decomp.coeffs)
  m=decomp.n1-decomp.n2
  n=decomp.n1+decomp.n2

  ; Perform dilatation
  if keyword_set(flux) then new_polar_coeffs=polar_coeffs*(1.-kappa) else  new_polar_coeffs=polar_coeffs*(1.+kappa)
  for i=0,decomp.n_coeffs-1 do begin

    j=where(m eq m(i) and n eq n(i)-2)
    if j(0) ne -1 then new_polar_coeffs(i) = new_polar_coeffs(i) + $
      ( kappa/2.*sqrt((n(i)-m(i))*(n(i)+m(i)))*polar_coeffs(j(0)) )

    j=where(m eq m(i) and n eq n(i)+2)
    if j(0) ne -1 then new_polar_coeffs(i) = new_polar_coeffs(i) - $
      ( kappa/2.*sqrt((n(i)-m(i)+2)*(n(i)+m(i)+2))*polar_coeffs(j(0)) )

  endfor

  ; Convert back to Cartesian shapelet coefficients
  decomp.coeffs=shapelets_p2c(new_polar_coeffs)


endif else begin

  ; Perform operation via a convolution with a delta-function

  alpha = decomp.beta     ; original beta
  beta  = alpha*1e-10     ; tiny! PSF is going to be a delta function
  gamma = alpha/(1+kappa) ; new beta

  shapelets_read_psf,psf,beta     ; create the delta function

  if not keyword_set(n_max) then n_max=decomp.n_max

  decomp=shapelets_convolve(decomp,psf,gamma=gamma,n_max_g=n_max)
  decomp.beta=decomp.beta*(1+kappa)

  ; Dilation should not conserve flux (should it be surface brightness?)
  ; The enlargement has just made it brighter, but by the wrong factor.
  decomp.coeffs=decomp.coeffs*(1+2*kappa)/(1.+kappa)

  ; Conserve flux if asked
  if keyword_set(flux) then decomp.coeffs=decomp.coeffs/scaling

endelse

end
