function shapelets_rsquared, decomp,        $
                             FLUX=flux,     $
                             MATRIX=matrix, $
                             ERROR=error,   $
                             POLAR=polar,   $
                             CARTESIAN=cartesian,rweight

;$Id: shapelets_rsquared.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;
;+
; NAME:
;      SHAPELETS_RSQUARED
;
; PURPOSE:
;      Computes the R^2 size measure from a linear summation of shapelet
;      coefficients, read in from a Cartesian decomp structure.
;
; CATEGORY:
;      Shapelets.
;
; INPUTS:
;      decomp - A Cartesian shapelet decomp structure, with at least
;               beta and coefficients defined.
;
; KEYWORD PARAMETERS:
;     [/CART] - Perform summation in Cartesian shapelet space. [DEFAULT]
;    [/POLAR] - Perform sunnation in polar shapelet space.
;    [/ERROR] - Simultaneously calculate the error on R^2 (and the flux).
;
; OUTPUTS:
;    rsquared - Shapelet rms size measure, squared.
;      [flux] - Floating point variable optionally returned with the object's 
;               flux (calculation needed to work out the centroid anyway).
;    [matrix] - Cartesian to polar conversion matrix, if /POLAR was set.
;
; EXAMPLE USE:
;      result = shapelets_rsquared(decomp,flux=flux)
;
; NOTES: 
;      Not yet compatible with oversampling, nor full covariance error matrix
;      of coefficients.
;
; MODIFICATION HISTORY:
;      Oct 03 - Calculation of errors added by RM.
;      Feb 03 - Polar/Cartesian switches added by RM. 
;      Apr 02 - Written by R.Massey
;-

COMPILE_OPT idl2

;
; SET DEFAULTS
;
if keyword_set(polar) then cartesian=0B else cartesian=1B


if cartesian then begin 
  ;
  ; WORK IN CARTESIAN SHAPELET SPACE  
  ;

  fweight=fltarr(decomp.n_coeffs)
  rweight=fltarr(decomp.n_coeffs)
  shapelets_make_nvec, decomp.n_max, n_1, n_2

  for i=0,decomp.n_coeffs-1 do begin
    n1=n_1[i] & n2=n_2[i]
    if n1 mod 2 eq 0 and n2 mod 2 eq 0 then begin

      fweight[i]=$
       2.^(.5*(2.-n1-n2)) * $
       sqrt(factorial(n1)*factorial(n2))/factorial(n1/2)/factorial(n2/2)

      rweight[i]=$
       2.^(.5*(4.-n1-n2)) * (1.+n1+n2) * $
       sqrt(factorial(n1)*factorial(n2))/factorial(n1/2)/factorial(n2/2)

    endif
  endfor

  rootpi   = sqrt(!pi)
  flux     = rootpi*decomp.beta*total(fweight*decomp.coeffs)
  rsquared = rootpi*decomp.beta^3/flux*total(rweight*decomp.coeffs)

  if keyword_set(error) then begin
    flux_error       = rootpi*decomp.beta*sqrt(total((fweight*decomp.error)^2))
    rsquared_variance= !pi*decomp.beta^6*total((rweight*decomp.error)^2)
    rsquared_error   = abs(rsquared)*$
                       sqrt( rsquared_variance/(rsquared*flux)^2 $
                            +(flux_error/flux)^2 )
    flux             = [flux,flux_error]
    rsquared         = [rsquared,rsquared_error]
  endif

endif else begin
  
  ;
  ; WORK IN POLAR SHAPELET SPACE 
  ;

  polar_coeffs=float(shapelets_c2p(decomp.coeffs,matrix=matrix))
  shapelets_make_nvec,n_max,n_r,n_l
  n=n_r+n_l
  m=n_r-n_l
  rootpi   = sqrt(!pi)
  i0=where(m eq 0)
  i1=where(m eq 1)
  i2=where(m eq 2)
  flux     = rootpi*decomp.beta*2*total(polar_coeffs[i0])
  rsquared = 4*rootpi*decomp.beta^3/flux*total((float(polar_coeffs*(1+n)))[i0])

  if keyword_set(error) then begin
    ; Errors will not be quite correct if we do not have the full covariance
    ;   matrix and try to perform the linear tranformation into polar space
    if size(decomp.error,/n_dimensions) ne 1 then message,$
      "Warning: error calculation using covariance matrix is not yet written!"
    polar_error=matrix#decomp.error
    flux_error=rootpi*decomp.beta*2*sqrt(total((float(polar_error)^2)[i0]))
    rsquaredf_variance=16*!pi*decomp.beta^6*total(((float(polar_error)*(1+n))^2)[i0])
    rsquared_error=rsquared*$
                   sqrt( rsquaredf_variance/(rsquared*flux)^2 $
                        +(flux_error/flux)^2 )

    flux=[flux,flux_error]
    rsquared=[rsquared,rsquared_error]
  endif

endelse

return,rsquared

end
