function shapelets_shapecat2decomp, shapecat, id, $
                                    OVER=over, INTEGRATE=integrate

;$Id: shapelets_shapecat2decomp.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_SHAPECAT2DECOMP
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Extracts one (decomp structure) object from a (shapecat structure)
;      catalogue.
;
; INPUTS:
;      shapecat - Shapelet catalogue structure. Can contain thousands of
;                 objects.
;      id       - If the catalogue contains more than one object, specify
;                 which one of them is to be converted.
;
; OPTIONAL INPUTS:
;      over     - Oversampling amount in decomp structure. Default: 1
;      integrate- Basis function integration in pixels.    Default: 1
;
; KEYWORD PARAMETERS:
;      none.
;
; OUTPUTS:
;      decomp   - Cartesian decomp structure containing all of the information
;                 that it is possible to reasonably extract from a catalogue.
;
; TO DO: 
;      Recover object's moments from shapelet catalogue.
;
; MODIFICATION HISTORY:
;      Mar 04 - Tidied up by RM.
;      Apr 02 - Can now take polar catalogues/objects as input
;      Dec 01 - Written by R.Massey
;-

on_error,2
n_pix_min=30
n_pix_max=100

if not keyword_set(id) then id=0
if id ge shapecat.n then message,"Object #"+strtrim(string(id),2)+" does not exist in this catalogue of only "+strtrim(string(shapecat.n),2)+" objects!"

n_max=shapecat.n_max[id]
shapelets_make_nvec,n_max,n1,n2,n_coeffs

;if shapecat.n eq 1 then begin
;
;  ; One-object catalogue (should really do this as only one case, using transpose first)
;  if cat.polar then begin
;   coeffs=shapelets_p2c(shapelets_polar_expand(transpose(shapecat.coeffs[0:n_coeffs-1]))) ; convert to cartesian
;    error=shapelets_p2c(shapelets_polar_expand(transpose(shapecat.error[0:n_coeffs-1])))
;  endif else begin
;    coeffs=transpose(shapecat.coeffs[0:n_coeffs-1])
;    error=transpose(shapecat.error[0:n_coeffs-1])
;  endelse
;
;endif else begin
;
;  ; Multi-object catalogue

  if shapecat.polar then begin
    coeffs=shapelets_p2c(shapelets_polar_expand(transpose(shapecat.coeffs[id,*]))) ; convert to cartesian
    coeffs=coeffs[0:n_coeffs-1]
    error=shapelets_p2c(shapelets_polar_expand(transpose(shapecat.error[id,*])))
    error=error[0:n_coeffs-1]
    ;print,(transpose(shapecat.coeffs))[0:n_coeffs-1,id]
    ;coeffs=reform(shapelets_p2c( (transpose(shapecat.coeffs))[0:n_coeffs-1,id],/EXPAND )) ; convert to cartesian
    ;error=reform(shapelets_p2c( (transpose(shapecat.error))[0:n_coeffs-1,id],/EXPAND ))
  endif else begin
    coeffs=reform((transpose(shapecat.coeffs))[0:n_coeffs-1,id])
    error=reform((transpose(shapecat.error))[0:n_coeffs-1,id])
  endelse
  
;endelse
beta=shapecat.beta[id]


; Rescale coefficients from relative to a_00 overall brightness
if n_coeffs gt 1 then coeffs(1:*)=coeffs(1:*)*coeffs(0)

; Guess (arbitrary) number of pixels for image display
fwhm=shapecat.sexfwhm[id]
th_max=beta*sqrt(n_max+0.5)
n_pixels=round([1.,1.]*1.5*th_max^1.1)+[10,10]
n_pixels(0)=max([n_pixels(0),n_pix_min]) & n_pixels(1)=max([n_pixels(1),n_pix_min])
n_pixels(0)=min([n_pixels(0),n_pix_max]) & n_pixels(1)=min([n_pixels(1),n_pix_max])
x=n_pixels/2.

; Recover sextractor information from shapelet catalogue
sex={seeing:shapecat.seeing,                                $
     x:[shapecat.sexx[id],shapecat.sexy[id]],               $
     xmin:[shapecat.objx_min[id],shapecat.objy_min[id]],    $
     xmax:[shapecat.objx_max[id],shapecat.objy_max[id]],    $
     id:shapecat.sexid[id],		                 		$
     class:shapecat.sexclass[id],	  			  		$
     a:shapecat.sexa[id], 		  			  		$
     b:shapecat.sexb[id], 		  			  		$
     theta:shapecat.sextheta[id],	  			  		$
     e:sqrt(shapecat.sexe1[id]^2+shapecat.sexe2[id]^2),     $
     e1:shapecat.sexe1[id],		  			     	$
     e2:shapecat.sexe2[id],		  			     	$
     flux:shapecat.sexflux[id],  	                    	$
     mag:shapecat.sexmag[id],  	  			     	$
     fwhm:shapecat.sexfwhm[id],	  			     	$
     area:shapecat.sexarea[id] }

; Recover object's moments from shapelet catalogue, to store in decomp.moments
;
; TO DO!
;

; Last bit of meta-information about the object's decomposition
if n_elements(over) eq 0      then over      = 1
if n_elements(integrate) eq 0 then integrate = 1

; store in a structure
decomp = {name:shapecat.name+"_"+strtrim(string(id)), $
          type:"cartesian_decomp",              $
          history:"Object #"+strtrim(string(id))+" in catalogue "+shapecat.name,       $
          x:x,                     		   $
          beta:beta,               		   $
          n_max:n_max,             		   $
          n_coeffs:n_coeffs,       		   $
          coeffs:coeffs,           		   $
          n_pixels:n_pixels,       		   $
          moments:0B,              		   $
          sex:sex,                 		   $
          n1:n1,                   		   $
          n2:n2,                   		   $
          over:over,               		   $
          integrate:integrate,     		   $
          error:error,             		   $
          chisq:[0.,0.],           		   $
          skyfit:0}

return,decomp

end
